#version 430
layout(location = 0) out vec3 color;

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

in vec3 toCameraVector;
in vec3 fromLightVector;
in float cameraHeight;  
in vec3 displacedPos;
  
uniform sampler2D reflectionTexture;
uniform sampler2D refractionTexture;
uniform sampler2D albedoMap;
uniform sampler2D metallicRoughnessMap;
uniform sampler2D aoMap;
uniform sampler2D normalMap;

uniform int debugCam;
uniform float moveFactor;

const float waveStrength = 0.01;
const float shineDamper = 20.0;
const float reflectivity = 0.2;
const float speed = 0.03f;

const float waterHeight = 13;

const vec3 blue = vec3(0.4, 0.94,0.87);

void main()
{
	vec3 norm = cameraHeight<waterHeight? vec3(0,-1.0,0):vec3(0,1.0,0);
	vec2 texbuffer = vec2(0,-0.08);
	
	vec2 texSize  = textureSize(refractionTexture, 0).xy;
    vec2 uv2 = (gl_FragCoord.xy)/texSize;
	
	
	vec2 refractTexCoords = uv2;
	vec2 reflectTexCoords = vec2(uv2.x,-uv2.y);
		
	//movements in water
	vec2 distortionTexCoords =  texture(metallicRoughnessMap,vec2(vert.uv.x+speed*moveFactor,vert.uv.y)).rg * 0.1;
	
	distortionTexCoords = vert.uv + vec2(distortionTexCoords.x,distortionTexCoords.y+speed*moveFactor);
	vec2 totalDistortion = (texture(metallicRoughnessMap,distortionTexCoords).rg * 2.0- 1.0)*waveStrength;

	vec2 refractTexCoordsDist = refractTexCoords+totalDistortion;
	refractTexCoordsDist = clamp(refractTexCoords,0.001,0.999);

	vec2 reflectTexCoordsDist = reflectTexCoords+totalDistortion;
	
	vec3 viewVector = normalize(toCameraVector);
	float refractiveFactor = dot(viewVector, norm);

	refractiveFactor = pow(refractiveFactor,2.5);
	

	vec4 reflectColor = texture(reflectionTexture,reflectTexCoordsDist);
	vec4 refractColor = texture(refractionTexture,refractTexCoordsDist);
	
	if (reflectColor.x<0.01 && reflectColor.y<0.01 && reflectColor.z<0.01)
	{
		reflectColor = texture(reflectionTexture,reflectTexCoords +texbuffer);
	}

	if (refractColor.x<0.01 && refractColor.y<0.01 && refractColor.z<0.01)
	{
		refractColor = texture(refractionTexture,refractTexCoords + texbuffer);
	}
	
	vec4 texMixture = mix(reflectColor, refractColor, refractiveFactor);

	//specular points, normalmap = normalMap
	vec4 normalMapColour = texture(normalMap,distortionTexCoords);
	vec3 normal = vec3(normalMapColour.r*2.0-1.0,normalMapColour.b,normalMapColour.g*2.0-1.0);
	normal = normalize(normal);
	
	vec3 reflectedLight = reflect(normalize(fromLightVector),normal);
	float specular = max(dot(reflectedLight,viewVector),0.4);
	specular = pow(specular,shineDamper);

	float dist = abs(displacedPos.y - vert.position_world.y);
	vec3 specularHighlights = vec3(1.0,1.0,1.0) * specular * reflectivity;
	
	vec4 higlight = vec4(specularHighlights+dist,1);
	vec4 tmp_color = mix(texMixture, vec4(blue,1), 0.2);
	color = mix(tmp_color, higlight,0.2).rgb;
	
 
}