#version 430 core

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec2 uv;

out VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

uniform mat4 modelMatrix;
uniform mat4 viewProjMatrix;
uniform mat3 normalMatrix;

out vec3 toCameraVector;
out vec3 fromLightVector;
out float cameraHeight; 
out vec3 displacedPos;

uniform vec3 lightPos;
uniform vec3 cameraPos;
uniform float u_time;
uniform sampler2D noiseTex;

void main() {

	vec3 pos = position;
	
	if (!(uv.x<0.02 || uv.y<0.02 || uv.x>0.98 || uv.y>0.98)) {
		float phase = u_time * 5;
		vec2 noiOff = vec2(0.8, 0.4) * u_time * 0.1;
		vec2 noiOff2 = vec2(0.2, 0.5) * u_time * 0.1;
		float hight1 = texture2D(noiseTex, uv + noiOff).r * 0.01;
		float hight2 = texture2D(noiseTex, uv + noiOff2).r * 0.02;
		float offset = (pos.x + (pos.z * hight1)) * 3.0;
		float offset2 = (pos.z + (pos.x * hight2)) * 8.0;
		float offset3 = (pos.z + (pos.x * hight1)) * 16.0;
		pos.y = (sin(phase + offset) + sin(phase + offset2)+ sin(phase + offset3)) * 0.01;
	
		float dx = pos.x;
		float dy = pos.y;
		float freq = sqrt(dx*dx + dy*dy);
		float amp = 0.02;
		float angle = -u_time*5.0+freq*10.0;
		pos.y += sin(angle)*amp;
	
	} 
	
	vec4 mvPosition = modelMatrix * vec4( pos, 1.0 );
	
	vert.normal_world = normalMatrix * normal;
	vert.uv = uv;
	
	vec4 position_world_ = mvPosition;
	vert.position_world = position_world_.xyz;
	
	toCameraVector = cameraPos - position_world_.xyz;
	fromLightVector= position_world_.xyz - lightPos;
	
	cameraHeight = cameraPos.y;
	displacedPos = (modelMatrix * vec4( position, 1.0 )).xyz;
	
	gl_Position = viewProjMatrix * mvPosition;
}