#version 430 core

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
} vert;

in float ttl;

layout(location = 0) out vec3 color;

uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform sampler2D diffuseTexture;
uniform float brightness;
uniform float maxTTL;

uniform bool toon;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
	vec3 color;
	vec3 position;
	vec3 attenuation;
} pointL;

const float levels = 3.0;

vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	float brightness =  max(0, dot(n, l));
	float specDamped = max(0, dot(r, v));
	if (toon) {
		float level = floor(brightness * levels);
		brightness = level / levels;
		level = floor(specDamped * levels);
		specDamped = level / levels;
	}
	return (diffuseF * diffuseC * brightness + specularF * specularC * pow(specDamped, alpha)) * att; 
}

void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	
	vec3 texColor = texture(diffuseTexture, vert.uv).rgb;
	float ambient = materialCoefficients.x * brightness;
	if (ambient > 1.0) {
		ambient = 1.0;
	}
	//float c = particles_in[gl_InstanceID].position.a;

	float normalizedTTL = ttl/maxTTL;
	texColor = vec3(1.0f, normalizedTTL	,0);
	color = vec3(texColor * ambient); // ambient
	
	// add directional light contribution
	color.rgb += phong(n, -dirL.direction, v, dirL.color * texColor, materialCoefficients.y, dirL.color, materialCoefficients.z, specularAlpha, false, vec3(0));
			
	// add point light contribution
	color.rgb += phong(n, pointL.position - vert.position_world, v, pointL.color * texColor, materialCoefficients.y, pointL.color, materialCoefficients.z, specularAlpha, true, pointL.attenuation);
	
	color.rgb = texColor;
}

