#version 430 core

in vec3 eye_vector;
in vec3 normal_world;

uniform sampler2DMS envMap;
uniform sampler2D backNormals;
uniform vec2 resolution;
uniform float ior;

out vec3 color;

int samples = 8;

float fresnel(vec3 direction, vec3 normal) {
    vec3 nDirection = normalize( direction );
    vec3 nNormal = normalize( normal );
    vec3 halfDirection = normalize( nNormal + nDirection );
    
    float cosine = dot( halfDirection, nDirection );
    float product = max( cosine, 0.0 );
    float factor = pow( product, 3.5 );
    
    return factor;
}

void main() {	
	
	color = vec3(0.0); 
	vec3 darkening = vec3(0.9);

	vec2 uv = gl_FragCoord.xy / resolution;

	
	vec3 backFaceNormal = texture(backNormals, uv).xyz;
	
	float a = 0.5;
	vec3 normal = normalize(normal_world * (1.0 - a) - backFaceNormal * a);
	vec3 dir = normalize(eye_vector);
	
	// Calculate refraction and add to the screen coordinates
	vec3 refracted = refract(dir, normal, 1.0/ior);
	uv += refracted.xy;

	// Manually wrap uv coordinates as this is not supported by multisampler
	
	if (uv.x > 1 || uv.x < 0) {
		if (mod(floor(uv.x), 2) == 0) {
			uv.x -= floor(uv.x);
		}
		else {
			uv.x = 1 - (uv.x - floor(uv.x));
		}
	}

	if (uv.y > 1 || uv.y < 0) {
		if (mod(floor(uv.y), 2) == 0) {
			uv.y -= floor(uv.y);
		}
		else {
			uv.y = 1 - (uv.y - floor(uv.y));
		}
	}

	ivec2 texSize = textureSize(envMap);
	ivec2 texCoord = ivec2(uv * texSize);

	
	for (int i = 0; i < samples; i++)
        color += texelFetch(envMap, texCoord, 1).xyz;

	color /= float(samples); 

	// Add fresnel effect
	float f = fresnel(dir, normal);

	color = mix(color, vec3(1.0), f);

	// Darkening due to light loss in glass
	color *= darkening;
			
}

